"""
==========================================================================
Kirigami Simulator
ABAQUS script for simulation of buckling-induced kirigami

Script written by Ahmad Rafsanjani
Last updated on 03 Dec 2019 
Contact info: ahmad.rafsanjani@gmail.com

Tested on ABAQUS 2019

To run the code in the Windows command:
abaqus cae nogui=kirigami.py

The output results are written in a CSV file
columns are as follows:
simulation time increment, strain x, strain y, stress x (MPa), stress y (MPa), 
elastic energy density (mJ/mm^3), spike's tip displacement (mm), tilting angle (degrees), 
compression plate displacement in z direction (mm), compression plate reaction force (N)

- StaticStep is recommended for tensile simulation
- ImplicitDynamicsStep is used for compression simulation

==========================================================================
"""


import numpy
import csv
import os
import glob

os.system("echo ============================================================")
os.system("echo  Kirigami Simulator")
os.system("echo  Script written by Ahmad Rafsanjani")
os.system("echo  Last updated on 03 Dec 2019")
os.system("echo  Contact info: ahmad.rafsanjani@gmail.com")



#   ------------------------------------------------------------------------
#   Solver

solver='StaticStep'    # select ['StaticStep', 'ImplicitDynamicsStep']
numCpus=8              # number of CPUs


#   ------------------------------------------------------------------------
#   Do you want to run compression test as well?

compression=False       # select [True, False] for running the compression step


#   ------------------------------------------------------------------------
#   Material Properties (stress unit is MPa)

E=193000.0   		# Young's modulus
nu=0.27    		    # Poisson's ratio 
rho=0.000008        # density (required for implicit dynamics analysis)
Yp=0#759.0			# Yield stress (set Yp to 0 for ignoring plasticty)


#   ------------------------------------------------------------------------
#   Geometry (dimension unit is mm)

shape='triangle'          # select [concave, convex, triangle]

inch2mm=25.4              # inch to mm conversion 
l=6.94                    # unit cell characteristic length 
d=0.1*l 				  # hinge width	
q=pi/6.0                  # angle 
tc=0.025                  # cut width
t=0.002*inch2mm           # sheet thickness
mesh_size=0.01*l          # mesh size
u_imp=0.005*l             # imperfection

L=2*l*cos(q)              # unit cell length in x direction
H=2*l*sin(q)			  # unit cell length in y direction	

Yz=0.5*H                  # location of compression plate (adjust it according to applied strain ey to be slightly above the spikes)

#   ------------------------------------------------------------------------
#   Loading

ey=0.15            # applied strain in tensile loading step
uz=-0.1*H          # displacement of compression plate in z direction
result_inc=0.01    # write output increments

#   ------------------------------------------------------------------------
#   Parameters

numpy.savetxt('parameters.dat', [l,d,q,t,tc,E,nu,rho,Yp,ey,mesh_size,u_imp,Yz,uz,result_inc])
shape_file = open("description.dat","w") 
shape_file.write(shape + "\n")
shape_file.write(str(compression)+ "\n")
shape_file.write(solver+ "\n")
shape_file.write(str(numCpus)+ "\n")
shape_file.close() 

#   ------------------------------------------------------------------------
#   Submit Jobs

os.system("echo ------------------------------------------------------------")
os.system("echo  shape={}, solver={}" .format(shape, solver))
os.system("echo  l={}mm, angle={}, d={}mm, t={}mm" .format(round(l,2), int(round(q*180/pi)),round(d,2),round(t,3)))

os.system("echo ------------------------------------------------------------")
os.system("echo  Linear Buckling Analysis")
os.system('ABAQUS CAE noGUI=buckling.py')


os.system("echo ------------------------------------------------------------")
if compression==True:
	os.system("echo  Nonlinear Post-Buckling Analysis + Compression")
else:
	os.system("echo  Nonlinear Post-Buckling Analysis")

os.system('ABAQUS CAE noGUI=postbuckling.py')


