"""
==========================================================================
Nonlinear Post-Buckling Analysis + Compression for Kirigami Simulator

Code: Ahmad Rafsanjani
Last update: 03 Dec 2019 
Gmail: ahmad.rafsanjani@gmail.com

Tested on ABAQUS 2019
==========================================================================
"""

#   ------------------------------------------------------------------------
#   Initialize simulation

from abaqus import *
import testUtils
testUtils.setBackwardCompatibility()
from abaqusConstants import *
from caeModules import *
import sketch
import part
from regionToolset import*
import load
from numpy import*
from math import *
from shutil import *
import numpy
from odbAccess import *
import assembly

session.journalOptions.setValues(replayGeometry=COORDINATE, recoverGeometry=COORDINATE)

#   ------------------------------------------------------------------------
#   Project 

project='postbuckling'

try:
    os.remove(project+'.lck')
    os.remove(project+'.odb')
except OSError:
    pass

model      = project+'_model'
sketch     = project+'_sketch'
material   = project+'_mat'
section    = project+'_sec'
cae_file   = project+'.cae'

#   ------------------------------------------------------------------------
#   Simulation Parameters

description_file = open("description.dat") 
shape=description_file.readline().rstrip('\n') 
compression=bool(description_file.readline().rstrip('\n')=='True')
solver=description_file.readline().rstrip('\n') 
numCpus=int(description_file.readline().rstrip('\n'))
description_file.close() 


elem_Code=S4R
[l,d,q,t,tc,E,nu,rho,Yp,ey,mesh_size,u_imp, Yz, uz, result_inc]=numpy.loadtxt('parameters.dat')


X0=l*cos(q)
Y0=l*sin(q)

L=2*X0
H=2*Y0
V0=L*H*t

k=6    # rounding digits

#   ------------------------------------------------------------------------
#   Functions

def sort_node_list(node_list,axis):
    """ Sorting a node list along a selected axis """
    node_list.sort(key=lambda node_list: node_list.coordinates[axis], reverse=False)


def sort_edge_list(edge_list,axis):
    edge_list.sort(key=lambda edge_list: edge_list.pointsOn[0][0][axis], reverse=False)


def ncoord(n,i):
    """ Returning the coordinate of a node in i axis """
    return round(n.coordinates[i],4)


def nappend(nset,n,i):
    nset.append(n.sequenceFromLabels(labels=(n[i].label,)))
    """ append a node to a node set"""

def periodic_bc(model,node_set1,node_set2,master,dof):

    num_nodes=len(node_set1)

    for i in range(num_nodes):
        n1=node_set1[i]
        n2=node_set2[i]


        n1_mask = ni.sequenceFromLabels(labels=(n1.label,))
        n2_mask = ni.sequenceFromLabels(labels=(n2.label,))
        
        n1_region = A.Set(nodes=n1_mask, name='n1_'+str(n1.label))
        n2_region = A.Set(nodes=n2_mask, name='n2_'+str(n2.label))


        for dof_i in dof:
            model.Equation(name='PBC_'+str(dof_i)+'_'+str(n1.label),
                           terms=(( 1.0 , 'n1_'+str(n1.label), dof_i),
                                  (-1.0 , 'n2_'+str(n2.label), dof_i),
                                  (-1.0, master, dof_i)))

    
#   ------------------------------------------------------------------------
#   Model 
m = mdb.Model(name=project)

s = m.ConstrainedSketch(name='s_'+project, sheetSize=2*L)
sp = m.ConstrainedSketch(name='sp_'+project, sheetSize=2*L)
st = m.ConstrainedSketch(name='st_'+project, sheetSize=2*L)

#   ------------------------------------------------------------------------
#   Geometry 

if shape=='triangle':

  x=zeros(16)
  y=zeros(16)


  x[0]=d*cos(q)-L/2.0
  y[0]=d*sin(q)-H/2.0

  x[1]=0.0
  y[1]=0

  x[2]=-d*cos(q)+L/2.0
  y[2]=d*sin(q)-H/2.0

  x[3]=-d*cos(q)+L/2.0-tc*sin(q)
  y[3]=d*sin(q)-tc*cos(q)-H/2.0

  x[4]=0.0
  y[4]=-tc/cos(q)

  x[5]=d*cos(q)-L/2.0+tc*sin(q)
  y[5]=d*sin(q)-tc*cos(q)-H/2.0


  x[6]=d*cos(q)
  y[6]=d*sin(q)


  x[7]=L/2.0
  y[7]=H/2.0

  x[8]=-L/2.0
  y[8]=H/2.0

  x[9]=-d*cos(q)
  y[9]=d*sin(q)

  x[10]=-d*cos(q)-tc*sin(q)
  y[10]=d*sin(q)-tc*cos(q)

  x[11]=-L/2.0
  y[11]=H/2.0-tc/cos(q)

  x[12]=-L/2.0
  y[12]=-H/2.0

  x[13]=L/2.0
  y[13]=-H/2.0

  x[14]=L/2.0
  y[14]=H/2.0-tc/cos(q)

  x[15]=d*cos(q)+tc*sin(q)
  y[15]=d*sin(q)-tc*cos(q)

  s.Line(point1=( x[0],  y[0]), point2=( x[1], y[1]))
  s.Line(point1=( x[1],  y[1]), point2=( x[2], y[2]))
  s.Line(point1=( x[3],  y[3]), point2=( x[4], y[4]))
  s.Line(point1=( x[4],  y[4]), point2=( x[5], y[5]))
  s.Line(point1=( x[2],  y[2]), point2=( x[3], y[3]))
  s.Line(point1=( x[0],  y[0]), point2=( x[5], y[5]))
  s.Line(point1=( x[8],  y[8]), point2=( x[9], y[9]))
  s.Line(point1=( x[10], y[10]), point2=(x[11], y[11]))
  s.Line(point1=( x[11], y[11]), point2=(x[12], y[12]))
  s.Line(point1=(-x[8],  y[8]), point2=(-x[9], y[9]))
  s.Line(point1=(-x[10], y[10]), point2=(-x[11], y[11]))
  s.Line(point1=(-x[11], y[11]), point2=(-x[12], y[12]))
  s.Line(point1=( x[9],  y[9]), point2=( x[10], y[10]))
  s.Line(point1=(-x[9],  y[9]), point2=(-x[10], y[10]))
  s.Line(point1=( x[13], y[13]), point2=( x[12], y[12]))
  s.Line(point1=( x[7],  y[7] ), point2=(  x[8], y[8] ))


elif shape=='convex':

  R=0.5*(l-d)/sin(q)

  x=zeros(8)
  y=zeros(8)

  x[0]=round(d*cos(q)-L/2.0,k)
  y[0]=round(d*sin(q)-H/2.0,k)

  x[1]=0
  y[1]=0

  x[2]=round(-d*cos(q)+L/2.0,k)
  y[2]=round(d*sin(q)-H/2.0,k)

  x[3]=round(-d*cos(q)+L/2.0-tc*sin(2*q),k)
  y[3]=round(d*sin(q)-tc*cos(2*q)-H/2.0,k)

  x[4]=0
  y[4]=-tc

  x[5]=round(d*cos(q)-L/2.0+tc*sin(2*q),k)
  y[5]=round(d*sin(q)-tc*cos(2*q)-H/2.0,k)


  x[6]=round(d*cos(q),k)
  y[6]=round(d*sin(q),k)

  x[7]=round(d*cos(q)+tc*sin(2*q),k)
  y[7]=round(d*sin(q)-tc*cos(2*q),k)


  s.Arc3Points(point1=(x[0],y[0]), point2=(x[2],y[2]), point3=(x[1],y[1]))
  s.Arc3Points(point1=(x[3],y[3]), point2=(x[5],y[5]), point3=(x[4],y[4]))
  s.Line(point1=( x[2], y[2]),point2=( x[3], y[3]))
  s.Line(point1=( x[0], y[0]),point2=( x[5], y[5]))
  s.ArcByCenterEnds(center=(L/2,H/2-R),point2=(L/2,H/2),point1=(x[6],y[6]),direction=CLOCKWISE)
  s.ArcByCenterEnds(center=(L/2,H/2-R),point2=(L/2,H/2-tc),point1=(x[7],y[7]),direction=CLOCKWISE)
  s.ArcByCenterEnds(center=(-L/2,H/2-R),point1=(-L/2,H/2),point2=(-x[6],y[6]),direction=CLOCKWISE)
  s.ArcByCenterEnds(center=(-L/2,H/2-R),point1=(-L/2,H/2-tc),point2=(-x[7],y[7]),direction=CLOCKWISE)
  s.Line(point1=( x[6], y[6]),point2=( x[7], y[7]))
  s.Line(point1=(-x[6], y[6]),point2=(-x[7], y[7]))
  s.Line(point1=(-L/2,H/2),point2=(L/2,H/2))
  s.Line(point1=(-L/2,-H/2),point2=(L/2,-H/2))
  s.Line(point1=(-L/2,H/2-tc),point2=(-L/2,-H/2))
  s.Line(point1=(L/2,H/2-tc),point2=(L/2,-H/2))

elif shape=='concave':


  R=0.5*(l-d)/sin(q)


  x=zeros(12)
  y=zeros(12)


  x[0]=d*cos(q)-L/2.0
  y[0]=d*sin(q)-H/2.0

  x[1]=0
  y[1]=0

  x[2]=-d*cos(q)+L/2.0
  y[2]=d*sin(q)-H/2.0

  x[3]=x[0]
  y[3]=y[0]-tc

  x[4]=0.0
  y[4]=-tc

  x[5]=d*cos(q)-L/2.0+tc*cos(q)
  y[5]=d*sin(q)-tc*sin(q)-H/2.0


  hc=sqrt((R+tc)*(R+tc)-x[0]*x[0])-(y[0]+R)


  x[6]=d*cos(q)
  y[6]=d*sin(q)

  x[7]=(d+tc)*cos(q)
  y[7]=(d-tc)*sin(q)


  x[8]=0
  y[8]=-hc


  x[9]=d*cos(q)
  y[9]=d*sin(q)

  x[10]=-x[9]
  y[10]=y[9]

  x[11]=L/2.0
  y[11]=H/2.0-hc


  s.ArcByCenterEnds(center=(x[0],y[0]+R),point2=(x[0],y[0]),point1=(x[1],y[1]),direction=CLOCKWISE)
  s.ArcByCenterEnds(center=(x[2],y[2]+R),point2=(x[1],y[1]),point1=(x[2],y[2]),direction=CLOCKWISE)

  s.ArcByCenterEnds(center=(x[0],y[0]+R),point2=(x[0],y[0]-tc),point1=(x[8],y[8]),direction=CLOCKWISE)
  s.ArcByCenterEnds(center=(x[2],y[2]+R),point2=(x[2],y[2]-tc),point1=(x[8],y[8]),direction=COUNTERCLOCKWISE)

  s.Line(point1=( x[0], y[0]),point2=( x[0], y[0]-tc))
  s.Line(point1=( x[2], y[2]),point2=( x[2], y[2]-tc))



  s.ArcByCenterEnds(center=(x[9],y[9]+R),point2=(x[9],y[9]),point1=(L/2.0,H/2.0),direction=CLOCKWISE)
  s.ArcByCenterEnds(center=(x[10],y[10]+R),point2=(x[10],y[10]),point1=(-L/2.0,H/2.0),direction=COUNTERCLOCKWISE)

  s.ArcByCenterEnds(center=(x[9],y[9]+R),point2=(x[9],y[9]-tc),point1=(x[11],y[11]),direction=CLOCKWISE)
  s.ArcByCenterEnds(center=(-x[9],y[9]+R),point2=(-x[9],y[9]-tc),point1=(-x[11],y[11]),direction=COUNTERCLOCKWISE)


  s.Line(point1=( x[9], y[9]),point2=( x[9], y[9]-tc))
  s.Line(point1=( -x[9], y[9]),point2=( -x[9], y[9]-tc))



  s.Line(point1=(-L/2,H/2),point2=(L/2,H/2))
  s.Line(point1=(-L/2,-H/2),point2=(L/2,-H/2))
  s.Line(point1=(-L/2,H/2-hc),point2=(-L/2,-H/2))
  s.Line(point1=(L/2,H/2-hc),point2=(L/2,-H/2))



sp.Line(point1=(0,H/2),point2=(0,-H/2))



p = m.Part(name=model, dimensionality=THREE_D, type=DEFORMABLE_BODY)
p.BaseShell(sketch=s)
p.PartitionFaceBySketch(faces=p.faces,sketch=sp)

px = m.Part(name='Ref_X', dimensionality=THREE_D, type=DEFORMABLE_BODY)
px.ReferencePoint(point=(X0, -Y0, 0))

py = m.Part(name='Ref_Y', dimensionality=THREE_D, type=DEFORMABLE_BODY)
py.ReferencePoint(point=(-X0, Y0, 0))

if compression==True:
  st.rectangle(point1=(-0.75*L, -0.75*L),point2=(0.75*L, 0.75*L))
  pt = m.Part(name='plate', dimensionality=THREE_D, type=DISCRETE_RIGID_SURFACE)
  pt.BaseShell(sketch=st)
  pt.ReferencePoint(point=(-0.75*L, -0.75*L, 0))

#   ------------------------------------------------------------------------
#   materials

mat= m.Material(material)
mat.Elastic(table=((E, nu), ))
mat.Density(table=((rho, ), ))
if Yp!=0:
  mat.Plastic(table=((Yp, 0.0),))

if compression==True:
  mat_rigid= m.Material(material+'_rigid')
  mat_rigid.Elastic(table=((1000*E, nu), ))

#   ------------------------------------------------------------------------
#   section

m.HomogeneousShellSection(idealization=NO_IDEALIZATION,
                          integrationRule=SIMPSON,
                          material=material,
                          name=section,
                          numIntPts=5,
                          poissonDefinition=DEFAULT,
                          preIntegrate=OFF,
                          temperature=GRADIENT,
                          thickness=t,
                          thicknessField='',
                          thicknessModulus=None,
                          thicknessType=UNIFORM,
                          useDensity=OFF)


p.SectionAssignment(offset=0.0,
                    offsetField='',
                    offsetType=MIDDLE_SURFACE,
                    region=Region(faces=p.faces),
                    sectionName=section,
                    thicknessAssignment=FROM_SECTION)

if compression==True:
  m.HomogeneousShellSection(idealization=NO_IDEALIZATION,
                            integrationRule=SIMPSON,
                            material=material+'_rigid',
                            name=section+'_rigid',
                            numIntPts=5,
                            poissonDefinition=DEFAULT,
                            preIntegrate=OFF,
                            temperature=GRADIENT,
                            thickness=t,
                            thicknessField='',
                            thicknessModulus=None,
                            thicknessType=UNIFORM,
                            useDensity=OFF)


  pt.SectionAssignment(offset=0.0,
                      offsetField='',
                      offsetType=MIDDLE_SURFACE,
                      region=Region(faces=pt.faces),
                      sectionName=section+'_rigid',
                      thicknessAssignment=FROM_SECTION)

#   ------------------------------------------------------------------------
#   Assembly


A = m.rootAssembly
I=A.Instance(dependent=ON, name='kirigami', part=p)
A.makeIndependent(instances=(I, ))
A.Instance(name='Ref_X', part=px, dependent=ON)
A.Instance(name='Ref_Y', part=py, dependent=ON)


if compression==True:
  IT=A.Instance(dependent=ON, name='rigid_plate', part=pt)
  A.makeIndependent(instances=(IT, ))
  A.translate(instanceList=('rigid_plate', ), vector=(0.0, 0.0,Yz))
  ST=A.Set(faces=IT.faces.findAt(((0, 0, Yz), )), name='rigid_surface')


A.regenerate()


#   ------------------------------------------------------------------------
#   Mesh


elem_type = mesh.ElemType(elemCode=elem_Code)
A.setElementType(regions=Region(faces=I.faces), elemTypes=(elem_type,))
A.seedEdgeByBias(biasMethod=DOUBLE, constraint=FIXED, endEdges=I.edges, maxSize=mesh_size*4, minSize=mesh_size)
A.setMeshControls(regions=I.faces, elemShape=QUAD_DOMINATED, algorithm=ADVANCING_FRONT)


if compression==True:
  A.setMeshControls(regions=IT.faces, elemShape=QUAD_DOMINATED)
  A.seedEdgeBySize(edges=IT.edges, constraint=FIXED, size=4*mesh_size)
  A.generateMesh(regions=(I,IT))
else:
  A.generateMesh(regions=(I,))

fi = I.faces
ei = I.edges
ni = I.nodes
vi = I.vertices
eli = I.elements



e_top=[]
e_bot=[]
e_left=[]
e_right=[]
e_cut=[]


xmin=-X0
xmax=X0
ymin=-Y0
ymax=Y0

for ej in ei:
    (xj,yj,zj)=ej.pointOn[0]

    if round(xj-xmin,4)==0:
        e_left.append(ei.findAt((ej.pointOn[0], ), ))
    elif round(xj-xmax,4)==0:
        e_right.append(ei.findAt((ej.pointOn[0], ), ))
    elif round(ymin-yj,4)==0:
        e_bot.append(ei.findAt((ej.pointOn[0], ), ))
    elif round(ymax-yj,4)==0:
        e_top.append(ei.findAt((ej.pointOn[0], ), ))
 
REFX=A.Set(name='RefX', referencePoints=(A.instances['Ref_X'].referencePoints[1],))
REFY=A.Set(name='RefY', referencePoints=(A.instances['Ref_Y'].referencePoints[1],))
if compression==True:
  REFT=A.Set(name='RefT',referencePoints=(A.instances['rigid_plate'].referencePoints[2],))

e_t=A.Set(edges=e_top,   name='e_top')
e_b=A.Set(edges=e_bot,   name='e_bot')
e_l=A.Set(edges=e_left,  name='e_left')
e_r=A.Set(edges=e_right, name='e_right')

n_top    = list(A.sets['e_top'].nodes)
n_bot    = list(A.sets['e_bot'].nodes)
n_right  = list(A.sets['e_right'].nodes)
n_left   = list(A.sets['e_left'].nodes)

sort_node_list(n_top,0)
sort_node_list(n_bot,0)
sort_node_list(n_right,1)
sort_node_list(n_left,1)



n_o=ni.sequenceFromLabels(labels=(n_left[0].label,))
N_O=A.Set(nodes=n_o, name='N_O')

n_tip=ni.sequenceFromLabels(labels=(n_left[-1].label,))
N_tip=A.Set(nodes=n_tip, name='N_tip')


#   ------------------------------------------------------------------------
#   Periodic Boundary Conditions

periodic_bc(m,n_top,  n_bot, 'RefY',[1,2,3,4,5,6])
periodic_bc(m,n_right,n_left,'RefX',[1,2,3,4,5,6])

#   ------------------------------------------------------------------------
#   Step

if solver=='ImplicitDynamicsStep':
	m.ImplicitDynamicsStep(name='Tensile',
	                       previous='Initial',
	                       description='ImplicitDynamicsStep',
	                       timePeriod=1.0,
	                       nlgeom=ON,
	                       initialInc=0.01,
	                       minInc=1e-9,
	                       maxNumInc=10000000,
	                       alpha=-0.4,
	                       amplitude=RAMP,
	                       application=MODERATE_DISSIPATION,
	                       initialConditions=OFF)

elif solver=='StaticStep':
	m.StaticStep(name='Tensile',
	             previous='Initial',
	             description='StaticStep',
	             timePeriod=1.0,
	             nlgeom=ON,
	             initialInc=result_inc*0.01,
	             minInc=result_inc*1e-6,
	             maxInc=result_inc,
	             maxNumInc=10000000,
	             stabilizationMethod=DISSIPATED_ENERGY_FRACTION,
	             stabilizationMagnitude=2e-4,
	             adaptiveDampingRatio=0.05)

if compression==True: #
	m.ImplicitDynamicsStep(name='Compression',
	                       previous='Tensile',
	                       description='ImplicitDynamicsStep',
	                       timePeriod=1.0,
	                       nlgeom=ON,
	                       initialInc=0.01,
	                       minInc=1e-9,
	                       maxNumInc=10000000,
	                       alpha=-0.4,
	                       amplitude=RAMP,
	                       application=MODERATE_DISSIPATION,
	                       initialConditions=OFF)



m.FieldOutputRequest(name='F-Output-1',
                     createStepName='Tensile',
                     timeInterval=result_inc,
                     variables=('U','S','E','PE','IVOL','RF','COORD'))

m.HistoryOutputRequest(name='H-Output-1', createStepName='Tensile', timeInterval=result_inc) 
                                             
#   ------------------------------------------------------------------------
#   Displacement BC

m.DisplacementBC(name='BC_O',
                 createStepName='Tensile',
                 region=N_O,
                 u1=0,
                 u2=0,
                 u3=0)

m.DisplacementBC(name='BC_REFX',
                 createStepName='Tensile',
                 region=REFX,
                 u2=0,
                 u3=0,
                 ur1=0.0,
                 ur2=0.0,
                 ur3=0.0)

m.DisplacementBC(name='BC_REFY',
                 createStepName='Tensile',
                 region=REFY,
                 ur1=0.0,
                 ur2=0.0,
                 ur3=0.0,
                 u1=0.0,
                 u2=ey*H,
                 u3=0.0)

if compression==True:
  m.DisplacementBC(name='BC_T',
                   createStepName='Tensile',
                   region=REFT,
                   ur1=0,
                   ur2=0,
                   ur3=0,
                   u1=0,
                   u2=0,
                   u3=0)
  m.boundaryConditions['BC_T'].setValuesInStep(stepName='Compression', u3=uz)


#   ------------------------------------------------------------------------
#   Contact
if compression==True:
	m.ContactProperty('IntProp')
	m.interactionProperties['IntProp'].NormalBehavior(allowSeparation=ON, constraintEnforcementMethod=DEFAULT, pressureOverclosure=HARD)
	m.interactionProperties['IntProp'].TangentialBehavior(formulation=FRICTIONLESS)
	m.ContactStd(createStepName='Initial', name='Int')
	m.interactions['Int'].includedPairs.setValuesInStep(stepName='Initial',useAllstar=ON)
	m.interactions['Int'].contactPropertyAssignments.appendInStep(assignments=((GLOBAL, SELF,'IntProp'), ), stepName='Initial')


#   ------------------------------------------------------------------------
#   Modify .inp file for imperfection

m.keywordBlock.synchVersions(storeNodesAndElements=True)
sieBlocks=m.keywordBlock.sieBlocks
Length_sieBlocks=len(m.keywordBlock.sieBlocks)



if solver=='ImplicitDynamicsStep':
	for i in range(Length_sieBlocks):
	    if sieBlocks[Length_sieBlocks-i-1]=='*Step, name=Tensile, nlgeom=YES, amplitude=RAMP, inc=10000000\nImplicitDynamicsStep':
	        kline=Length_sieBlocks-i
	        break
elif solver=='StaticStep':
	for i in range(Length_sieBlocks):
	    if sieBlocks[Length_sieBlocks-i-1]=='*Step, name=Tensile, nlgeom=YES, inc=10000000\nStaticStep':
	        kline=Length_sieBlocks-i
	        break

m.keywordBlock.insert(kline-3,'\n*IMPERFECTION, FILE=buckling, STEP=1\n1,%f'% u_imp)


#   ------------------------------------------------------------------------
#   Job

j=mdb.Job(name=project,
          model=project,
          numCpus=numCpus,
          numDomains=numCpus,
          multiprocessingMode=DEFAULT,
          description='Post-Buckling Analysis')
mdb.saveAs(cae_file)
j.submit()
j.waitForCompletion()


#   ------------------------------------------------------------------------
#   Post-processing

os.system("echo ------------------------------------------------------------")
os.system("echo  Postprocessing...")

odb = openOdb(project+'.odb')
session.viewports['Viewport: 1'].setValues(displayedObject=odb)
REFX_data=session.xyDataListFromField(odb=odb,
                                   outputPosition=NODAL,
                                   variable=(('COORD', NODAL,
                                              ((COMPONENT, 'COOR1'),
                                               (COMPONENT, 'COOR2'), )),
                                             ('RF', NODAL,
                                              ((COMPONENT, 'RF1'),
                                               (COMPONENT, 'RF2'), )),
                                             ('U', NODAL,
                                              ((COMPONENT, 'U1'),
                                               (COMPONENT, 'U2'), )), ),
                                   nodeSets=('REFX', ))


REFY_data=session.xyDataListFromField(odb=odb,
                                   outputPosition=NODAL,
                                   variable=(('COORD', NODAL,
                                              ((COMPONENT, 'COOR1'),
                                               (COMPONENT, 'COOR2'), )),
                                             ('RF', NODAL,
                                              ((COMPONENT, 'RF1'),
                                               (COMPONENT, 'RF2'), )),
                                             ('U', NODAL,
                                              ((COMPONENT, 'U1'),
                                               (COMPONENT, 'U2'), )), ),
                                   nodeSets=('REFY', ))
if compression==True:
	REFT_data=session.xyDataListFromField(odb=odb,
	                                   outputPosition=NODAL,
	                                   variable=(('COORD', NODAL,
	                                              ((COMPONENT, 'COOR1'),
	                                               (COMPONENT, 'COOR2'),
	                                               (COMPONENT, 'COOR3'), )),
	                                             ('RF', NODAL,
	                                              ((COMPONENT, 'RF1'),
	                                               (COMPONENT, 'RF2'),
	                                               (COMPONENT, 'RF3'), )),
	                                             ('U', NODAL,
	                                              ((COMPONENT, 'U1'),
	                                               (COMPONENT, 'U2'),
	                                               (COMPONENT, 'U3'), )), ),
	                                   nodeSets=('REFT', ))

N_TIP=session.xyDataListFromField(odb=odb,
                            outputPosition=NODAL,
                            variable=(('U', NODAL,
                                       ((COMPONENT, 'U1'),
                                        (COMPONENT, 'U2'),
                                        (COMPONENT, 'U3'), )),
                                      ('COORD', NODAL,
                                       ((COMPONENT, 'COOR1'),
                                        (COMPONENT, 'COOR2'),
                                        (COMPONENT, 'COOR3'),)), ),
                            nodeSets=('N_TIP',))

ALLSE=session.XYDataFromHistory(odb=odb,
                             name='ALLSE',
                             outputVariableName='Strain energy: ALLSE for Whole Model',
                             steps=('Tensile', 'Compression', ), )



N=len(N_TIP[0])
# Define array for outputs
if compression==True:
	RES=numpy.zeros((N,10)) 
else: 
	RES=numpy.zeros((N,8))  

U_tip=[(N_TIP[0][i][1],N_TIP[1][i][1],N_TIP[2][i][1]) for i in range(N)]


for i in range(1,N):
 RES[i][0]= tn   =REFX_data[0][i][0]                    # simulation time increment
 RES[i][1]= exx  =REFX_data[4][i][1]/L				    # strain in x direction
 RES[i][2]= eyy  =REFY_data[5][i][1]/H				    # strain in y direction
 RES[i][3]= Sx   = REFX_data[2][i][1]/(H*t)			    # stress in x direction in MPa (should be zero)
 RES[i][4]= Sy   = REFY_data[3][i][1]/(L*t)			    # stress in y direction in MPa
 RES[i][5]= ALLSE[i][1]/V0								# strain energy density in mJ/mm^3
 RES[i][6]= N_TIP[2][i][1]								# displacement of the tip of the spike in mm
 RES[i][7]= abs(numpy.arctan2(N_TIP[5][i][1],(0.5*H+N_TIP[4][i][1]))*180/pi) # tilting angle in degrees
 if compression==True:
	 RES[i][8]= uz   =REFT_data[8][i][1]				# displacement of compression plate in z direction in mm
	 RES[i][9]= Fz   =REFT_data[5][i][1]				# reaction force of compression plate in z direction in N

numpy.savetxt('results_'+shape+'.csv', RES, fmt='%6.12f',delimiter=',')

os.system("echo ============================================================")
